@@ -531,6 +531,19 @@ for filt in ['Red']:
     _BANDPASSES.register_loader(name, load_bandpass_remote_aa,
                                 args=(relpath,), meta=tess_meta)
 
+# GALEX
+galex_meta = {
+    'filterset': 'galex',
+    'retrieved': '10 March 2025',
+    'dataurl': ('http://svo2.cab.inta-csic.es/svo/theory/fps/getdata.php?'
+                'format=ascii&id=Misc'),
+    'description': 'GALEX filters from SVO (includes filter and instrument)'
+}
+for filt in ['fuv', 'nuv']:
+    name = 'galex::' + filt
+    relpath = 'bandpasses/galex/galex.{}'.format(filt)
+    _BANDPASSES.register_loader(name, load_bandpass_remote_aa,
+                                args=(relpath,), meta=galex_meta)
 
 # GOTO
 goto_meta = {
