@@ -59,6 +59,10 @@ def generate_chisq(data, model, spectra, signature='iminuit', modelcov=False):
                                             zp=data.zp, zpsys=data.zpsys)
                 diff = data.flux - model_flux
                 phot_chisq = np.dot(np.dot(diff, invcov), diff)
+                if math.isnan(phot_chisq):
+                    # Tell iminuit to keep away from this point
+                    # (in most cases this should be inf instead of nan anyway)
+                    phot_chisq = float('inf')
                 full_chisq += phot_chisq
 
             if spectra is not None:
